require 'spec_helper'
require 'evrone/ci/message/testing'

describe Evrone::CI::SCM::Git do
  let(:path)       { '/tmp/.test/repo'               }
  let(:message)    { Evrone::CI::Message::PerformBuild.test_message }
  let(:src)        { message.src }
  let(:sha)        { message.sha }
  let(:deploy_key) { message.deploy_key }
  let(:output)     { "" }
  let(:git)        {
    described_class.new src, sha, path, deploy_key: deploy_key, &method(:add_to_output)
  }

  subject { git }

  before { FileUtils.rm_rf path }
  after  { FileUtils.rm_rf path }

  context "just created" do
    its(:src)                 { should eq src        }
    its(:sha)                 { should eq sha        }
    its(:path)                { should eq path       }
    its("git_ssh.deploy_key") { should eq deploy_key }
  end

  context "fetch" do
    subject { git.fetch }

    it { should eq 0 }

    it "should create nessesary directories and checkout sha" do
      subject
      expect(File.directory? path).to be
      expect(File.directory? "#{path}/.git").to be
      Dir.chdir path do
        expect((`git rev-parse HEAD`).strip).to eq sha
      end
    end

    it "should capture output" do
      subject
      expect(output).to eq "$ git clone -q #{src} #{path}\n$ git checkout -qf #{sha}\n"
    end

    context "twice" do
      before { git.fetch }
      it { should eq 0 }
    end

    context "with error" do
      let(:src) { "/not-exists-repo.git"  }

      it "should return 128 exitstatus and add error to output" do
        expect(subject).to eq 128
        expect(output).to match('does not exist')
      end
    end
  end

  context "make_fetch_command" do
    include Evrone::Common::Spawn
    let(:env) { {'GIT_SSH' => git.git_ssh.location.path} }
    let(:run) do
      git.git_ssh.open do
        spawn(env, git.make_fetch_command) {|o| puts o }
      end
    end
    subject { git.make_fetch_command }

    before do
      run
    end

    it { should eq "(test -d #{path} || git clone -q #{src} #{path}) && cd #{path} && git checkout -qf #{sha}" }

    it "should be success" do
      expect(run).to eq 0
    end

    it "should create nessesary directories and checkout sha" do
      expect(File.directory? path).to be
      expect(File.directory? "#{path}/.git").to be
      Dir.chdir path do
        expect((`git rev-parse HEAD`).strip).to eq sha
      end
    end

    context "twice" do
      it "should be" do
        code = git.git_ssh.open do
          spawn(env, git.make_fetch_command) {|o| puts o }
        end
        expect(code).to eq 0
      end
    end

    context "with error" do
      let(:src) { "/not-exists-repo.git"  }

      it "should return 128 exitstatus and add error to output" do
        expect(run).to eq 128
      end
    end
  end

  context ".make_export_command" do
    let(:from) { path }
    let(:to)   { '/tmp/.test/export' }
    let(:expected) { "(cd '#{from}' && git checkout-index -a -f --prefix='#{to}/')" }
    subject { described_class.make_export_command from, to}
    it { should eq expected }

    context "run" do
      before do
        git.fetch
        system subject
      end

      it "should be success" do
        expect($?.to_i).to eq 0
      end

      it "should export repo" do
        expect(File.readable? "#{to}/Gemfile").to be_true
      end
    end
  end

  context "commit_info" do
    subject { git.commit_info }
    before  { git.fetch       }

    it "should be" do
      expect(subject.sha).to     eq 'b665f90239563c030f1b280a434b3d84daeda1bd'
      expect(subject.author).to  eq "Dmitry Galinsky"
      expect(subject.email).to   eq 'dima.exe@gmail.com'
      expect(subject.message).to eq 'first release'
    end
  end

  def add_to_output(out)
    puts "==> #{out}"
    output << out
  end
end
